@@ -99,16 +99,16 @@ def test_basic(tmpdir, capsys):
     assert 'cannot find dictionary' in stderr
     os.remove(fname)
 
-    with open(op.join(d, 'bad.txt'), 'w') as f:
-        f.write('abandonned\nAbandonned\nABANDONNED\nAbAnDoNnEd')
-    assert cs.main(d) == 4
+    with open(op.join(d, 'bad.txt'), 'w', newline='') as f:
+        f.write('abandonned\nAbandonned\nABANDONNED\nAbAnDoNnEd\nabandonned\rAbandonned\r\nABANDONNED \n AbAnDoNnEd')  # noqa: E501
+    assert cs.main(d) == 8
     code, _, stderr = cs.main('-w', d, std=True)
     assert code == 0
     assert 'FIXED:' in stderr
-    with open(op.join(d, 'bad.txt')) as f:
+    with open(op.join(d, 'bad.txt'), newline='') as f:
         new_content = f.read()
     assert cs.main(d) == 0
-    assert new_content == 'abandoned\nAbandoned\nABANDONED\nabandoned'
+    assert new_content == 'abandoned\nAbandoned\nABANDONED\nabandoned\nabandoned\rAbandoned\r\nABANDONED \n abandoned'  # noqa: E501
 
     with open(op.join(d, 'bad.txt'), 'w') as f:
         f.write('abandonned abandonned\n')
@@ -272,6 +272,19 @@ def test_encoding(tmpdir, capsys):
     with open(f.name, 'ab') as f:
         f.write(u'naieve\n'.encode('utf-8'))
     assert cs.main(f.name) == 1
+    # Encoding detection (only try ISO 8859-1 because UTF-8 is the default)
+    with open(f.name, 'wb') as f:
+        f.write(b'Speling error, non-ASCII: h\xe9t\xe9rog\xe9n\xe9it\xe9\n')
+    # check warnings about wrong encoding are enabled with "-q 0"
+    code, stdout, stderr = cs.main('-q', '0', f.name, std=True, count=True)
+    assert code == 1
+    assert 'Speling' in stdout
+    assert 'iso-8859-1' in stderr
+    # check warnings about wrong encoding are disabled with "-q 1"
+    code, stdout, stderr = cs.main('-q', '1', f.name, std=True, count=True)
+    assert code == 1
+    assert 'Speling' in stdout
+    assert 'iso-8859-1' not in stderr
     # Binary file warning
     with open(f.name, 'wb') as f:
         f.write(b'\x00\x00naiive\x00\x00')
@@ -393,6 +406,63 @@ def test_case_handling(tmpdir, capsys):
         assert f.read().decode('utf-8') == 'this has an ASCII error'
 
 
+def _helper_test_case_handling_in_fixes(tmpdir, capsys, reason):
+    d = str(tmpdir)
+
+    with open(op.join(d, 'dictionary.txt'), 'w') as f:
+        if reason:
+            f.write('adoptor->adopter, adaptor, reason\n')
+        else:
+            f.write('adoptor->adopter, adaptor,\n')
+    dictionary_name = f.name
+
+    # the mispelled word is entirely lowercase
+    with open(op.join(d, 'bad.txt'), 'w') as f:
+        f.write('early adoptor\n')
+    code, stdout, _ = cs.main('-D', dictionary_name, f.name, std=True)
+    # all suggested fixes must be lowercase too
+    assert 'adopter, adaptor' in stdout
+    # the reason, if any, must not be modified
+    if reason:
+        assert 'reason' in stdout
+
+    # the mispelled word is capitalized
+    with open(op.join(d, 'bad.txt'), 'w') as f:
+        f.write('Early Adoptor\n')
+    code, stdout, _ = cs.main('-D', dictionary_name, f.name, std=True)
+    # all suggested fixes must be capitalized too
+    assert 'Adopter, Adaptor' in stdout
+    # the reason, if any, must not be modified
+    if reason:
+        assert 'reason' in stdout
+
+    # the mispelled word is entirely uppercase
+    with open(op.join(d, 'bad.txt'), 'w') as f:
+        f.write('EARLY ADOPTOR\n')
+    code, stdout, _ = cs.main('-D', dictionary_name, f.name, std=True)
+    # all suggested fixes must be uppercase too
+    assert 'ADOPTER, ADAPTOR' in stdout
+    # the reason, if any, must not be modified
+    if reason:
+        assert 'reason' in stdout
+
+    # the mispelled word mixes lowercase and uppercase
+    with open(op.join(d, 'bad.txt'), 'w') as f:
+        f.write('EaRlY AdOpToR\n')
+    code, stdout, _ = cs.main('-D', dictionary_name, f.name, std=True)
+    # all suggested fixes should be lowercase
+    assert 'adopter, adaptor' in stdout
+    # the reason, if any, must not be modified
+    if reason:
+        assert 'reason' in stdout
+
+
+def test_case_handling_in_fixes(tmpdir, capsys):
+    """Test that the case of fixes is similar to the mispelled word."""
+    _helper_test_case_handling_in_fixes(tmpdir, capsys, reason=False)
+    _helper_test_case_handling_in_fixes(tmpdir, capsys, reason=True)
+
+
 def test_context(tmpdir, capsys):
     """Test context options."""
     d = str(tmpdir)
@@ -734,35 +804,55 @@ def test_uri_regex_def():
         assert not uri_regex.findall(boilerplate % uri), uri
 
 
-def test_config(tmpdir, capsys):
-    """
-    Tests loading options from a config file.
-    """
-    d = str(tmpdir)
-
-    # Create sample files.
-    with open(op.join(d, 'bad.txt'), 'w') as f:
+@pytest.mark.parametrize('kind', ('toml', 'cfg'))
+def test_config_toml(tmp_path, capsys, kind):
+    """Test loading options from a config file or toml."""
+    d = tmp_path / 'files'
+    d.mkdir()
+    with open(d / 'bad.txt', 'w') as f:
         f.write('abandonned donn\n')
-    with open(op.join(d, 'good.txt'), 'w') as f:
+    with open(d / 'good.txt', 'w') as f:
         f.write("good")
 
-    # Create a config file.
-    conffile = op.join(d, 'config.cfg')
-    with open(conffile, 'w') as f:
-        f.write(
-            '[codespell]\n'
-            'skip = bad.txt\n'
-            'count = \n'
-        )
-
     # Should fail when checking both.
-    code, stdout, _ = cs.main(d, count=True, std=True)
+    code, stdout, _ = cs.main(str(d), count=True, std=True)
     # Code in this case is not exit code, but count of misspellings.
     assert code == 2
     assert 'bad.txt' in stdout
 
+    if kind == 'cfg':
+        conffile = str(tmp_path / 'setup.cfg')
+        args = ('--config', conffile)
+        with open(conffile, 'w') as f:
+            f.write("""\
+[codespell]
+skip = bad.txt, whatever.txt
+count =
+""")
+    else:
+        assert kind == 'toml'
+        pytest.importorskip('tomli')
+        tomlfile = str(tmp_path / 'pyproject.toml')
+        args = ('--toml', tomlfile)
+        with open(tomlfile, 'w') as f:
+            f.write("""\
+[tool.codespell]
+skip = 'bad.txt,whatever.txt'
+count = false
+""")
+
     # Should pass when skipping bad.txt
-    code, stdout, _ = cs.main('--config', conffile, d, count=True, std=True)
+    code, stdout, _ = cs.main(str(d), *args, count=True, std=True)
+    assert code == 0
+    assert 'bad.txt' not in stdout
+
+    # And both should automatically work if they're in cwd
+    cwd = os.getcwd()
+    try:
+        os.chdir(tmp_path)
+        code, stdout, _ = cs.main(str(d), count=True, std=True)
+    finally:
+        os.chdir(cwd)
     assert code == 0
     assert 'bad.txt' not in stdout
 
